<?php

namespace Drupal\entity_access\Tests;

use Drupal\simpletest\WebTestBase;

/**
 * Class EntityAccessTest.
 *
 * @group entity_access
 */
class EntityAccessTest extends WebTestBase {

  /**
   * {@inheritdoc}
   */
  public static $modules = ['block', 'entity_access_test'];

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    $this->drupalLogin($this->createUser(['administer content types']));
    $this->drupalPlaceBlock('local_tasks_block');
  }

  /**
   * Tests access to the entity with specified bundle condition.
   */
  public function testEntityBundleAccess() {
    foreach (['page' => 403, 'article' => 200] as $type => $response) {
      $path = "admin/structure/types/manage/$type";

      $this->drupalCreateContentType(['type' => $type]);

      // Visit page with form for content type editing.
      $this->drupalGet($path);
      $this->assertResponse(200);

      // Check existence of local task's tab: it have not be on
      // the page if page is not accessible.
      $this->{200 === $response ? 'assertLink' : 'assertNoLink'}('Test task');

      // Ensure that page is not accessible.
      $this->drupalGet("$path/test");
      $this->assertResponse($response);
    }
  }

}
